let TF_ImportImages;
((Themify,doc,win,Promise,und) =>{
    'use strict';
    TF_ImportImages={
        downloaded:new Map,
        uploading:new Set,
        _getImage(url,timer){
            return new Promise((resolve,reject)=>{
                setTimeout(()=>{
                    this._download(url)
                    .then(resolve)
                    .catch(reject);
                },timer);
            });
        },
        async _download(url){
            if(typeof url!=='string'){
                return url;
            }
            const blob=this.downloaded.get(url);
            if(blob!==und){
                return blob;
            }
            if(!url.includes('themify.me')&& !url.includes('themify.org')){
                return 1;
            }
            try{
                return await Themify.fetch('', 'blob', {
                    credentials:'omit',
                    method:'GET',
                    mode:'cors',
                    keepalive:true,
                    priority:'high'
                }, url+'?polish=0');
            }
            catch(e){
                if(e.status===404){
                    return false;
                }
                else{
                    return 1;//1 means try to download in the backend
                }
            }
        },
        async _batchDownload(images,timer){
            const batch=()=>{
                    const prms=[];
                    for(let i=0;i<images.length;++i){
                       prms.push(this._getImage(images[i],timer));
                    }
                    return Promise.all(prms);
                };
            
            let res;
            try{
                res=await batch();
            }
            catch(e){
                try{
                    res=await batch();//try again
                }
                catch(e){
                    res=Array(images.length).fill(1);//try to download in the backend
                }
            }
            return res;
        },
        async _batchUpload(formData,timer){
            let response=null, 
            postsData=JSON.parse(formData.get('postData')),
            _upload=formData=>{
                return new Promise((resolve,reject)=>{
                    setTimeout(()=>{
                        Themify.fetch(formData).then(res=>{
                            if(res?.success){
                                resolve(res);
                            }
                            else{
                                reject(res.data);
                            }
                        })
                        .catch(reject);
                    },timer);
                });
            },
            responseHandler=respData=>{
                for(let i in postsData){
                    let url=postsData[i].thumb;
                    if(respData[url]?.id){
                        delete postsData[i];
                        formData.delete(i);
                    }
                    response.data[url]=respData[url];
                }
            };
            for(let i in postsData){
                if(!this.uploading.has(postsData[i].thumb)){
                    this.uploading.add(postsData[i].thumb);
                }else{
                    delete postsData[i];
                    formData.delete(i);
                }
            }
            if(Object.keys(postsData).length>0){
                try{
                    response={data:{}};
                    formData.set('postData',JSON.stringify(postsData));
                    const res=await _upload(formData);
                    response.success=true;
                    responseHandler(res.data);
                    if(Object.keys(postsData).length>0){
                        formData.set('postData',JSON.stringify(postsData));
                        throw '';
                    }
                }
                catch(e){
                    try{
                        const res=await _upload(formData);//try again
                        responseHandler(res.data);
                        if(Object.keys(postsData).length>0){
                            formData.set('postData',JSON.stringify(postsData));
                            throw '';
                        }
                    }
                    catch(e){
                        try{
                            for(let i in postsData){
                                formData.delete(i);
                                if(postsData[i].thumb.indexOf('http')!==0){
                                    delete postsData[i];
                                }
                            }
                            if(Object.keys(postsData).length>0){
                                const res=await _upload(formData);//try again without blob in backend
                                responseHandler(res.data);
                            }
                        }
                        catch(e){

                        }
                    }
                }
            }
            return response;
        },
        init(images,nonce,messages,chunkSize=2,action='themify_upload_image'){
            return new Promise(async resolve=>{
                if(!Array.isArray(images)){
                    images=[...images];
                }
                let isVisible=true,
                    isSend1=false,
                    isSend2=false,
                    summ=0,
                    errors=new Map;
                await Themify.loadJs(Themify.url+'js/admin/notification',!!win.TF_Notification);
                const len=images.length,
                    downLoadMsg=messages.download_images?messages.download_images.replaceAll('%to%',len):'',
                    upLoadMsg=messages.upload_images?messages.upload_images.replaceAll('%to%',len):'',
                    download_fail=messages.download_fail || '',
                    upload_fail=messages.upload_fail || '',
                    customData=messages.custom_data || '',
                    stop_webp=messages.stop_webp || '',
                    resp=new Map,
                    timer=len>12?150:100,
                    prms=[],
                    _getImagesUrls=images=>{
                        const arr=[];
                        for(let i=0;i<images.length;++i){
                            let url=images[i];
                            if(!url.size){//instanceof File doesn't work
                                url=typeof url==='string'?url.trim():url.url.trim();
                                if(url.includes('themes/ultra-agency-2/')){
                                   url=url.replace('/themes/ultra-agency-2/files/','/themes/ultra-agency2/files/');
                                }
                            }
                            arr.push(url);
                        }
                        return arr;
                    },
                    downLoadNext=images=>{
                        images=_getImagesUrls(images);
                        for(let i=images.length-1;i>-1;--i){
                           
                            this._download(images[i]).then(b=>{
                                this.downloaded.set(images[i],b); 
                            });
                        }
                    },
                    preventReload=e=>{
                        e.preventDefault();
                        return e.returnValue='Are you sure';
                    },
                    visibilitychange=()=>{
                        isVisible=win.document.visibilityState==='visible';
                    },
                    _chunks=(items,size)=>{
                        const res=[];
                        while(items.length > 0) {
                            res.push(items.splice(0, size));
                        }
                        return res;
                    },
                    _prepareUpload=(images,arrBlob)=>{
                        const formData = new FormData(),
                        postsData={},
                        skip=new Set;
                        for(let i=arrBlob.length-1;i>-1;--i){
                            if(images[i]!==und){
                                let item=images[i],
                                    isString=typeof item==='string',
                                    url=item.name??(item.url??item),
                                    blob=arrBlob[i];
                                if(blob!==false && blob!=='skip'){
                                    formData.set(i,blob);
                                    postsData[i]={
                                        thumb:url
                                    };
                                    if(isString===false){
                                        if(item.id){
                                            postsData[i].post_id=item.id;
                                        }
                                        if(item.term_id){
                                            postsData[i].term_id=item.term_id;
                                        }
                                        if(item.data){
                                            postsData[i].data=item.data;
                                        }
                                    }
                                }
                                else{
                                    skip.add(url);
                                    resp.set(url,false);  
                                }
                            }
                        }
                        formData.set('action',action);
                        formData.set('nonce',nonce);
                        formData.set('postData',JSON.stringify(postsData));
                        if(customData){
                            formData.set('save_id',customData);
                        }
                        if(stop_webp){
                            formData.set('stop_webp',1);
                        }
                        return {formData:formData,skip:skip};
                    },
                    _uploadFinall=(r,images)=>{
                        if(!r?.success){
                            if(!r){
                                r='';
                            }
                            else if(r.data){
                                r=r.data;
                            }
                        }
                        else if(r.data){
                            r=r.data;
                        }
                        const result=new Map;
                        for(let i=images.length-1;i>-1;--i){
                            let url=images[i].name??(images[i].url??images[i]),
                                v=this.downloaded.get(url);
                            if(v!==false && v!=='skip'){
                                this.downloaded.delete(url);
                            }
                            if(v!==false && v!=='skip' &&  r?.[url]?.id){
                                resp.set(url,r[url]);
                            }
                            else{
                                resp.set(url,false);
                                let msg= r?.[url] || '';
                                result.set(url,msg);
                            }
                        }
                        return result;
                        
                    };
                    win.tfOn('beforeunload',preventReload)
                    .document.tfOn('visibilitychange', visibilitychange, {passive:true});
            
                    images=_chunks(images,chunkSize); 
                    wp?.heartbeat?.interval(120);
                    prms[0]=new Promise(async resolve =>{
                        let blobs=[];
                    
                        for(let i=0,arrLen=images.length;i<arrLen;++i){
                            try{
                                let res=[],
                                    imgArr=_getImagesUrls(images[i]),
                                    found=false;
                                    summ+=imgArr.length;
                                for(let j=imgArr.length-1;j>-1;--j){
                                    let data=resp.get(imgArr[j]);
                                    if(!data){
                                        found=true;
                                        break;
                                    }
                                    else{
                                        res[j]=data;
                                    }
                                }
                                if(found===true){
                                    isSend1=false;
                                    if(isVisible===true){
                                        let f=doc.createDocumentFragment(),
                                            msg=doc.createElement('div'),
                                            img_wrap=doc.createElement('div'),
                                            hasDownLoad=false;
                                            img_wrap.className='img_wrap';
                                            msg.className='msg_text';
                                        for(let j=blobs.length-1;j>-1;--j){
                                            URL.revokeObjectURL(blobs[j]);
                                        }
                                        blobs=[];
                                        for(let j=imgArr.length-1;j>-1;--j){
                                            let src=imgArr[j],
                                                img=new Image(50,50);
                                            if(src.type){
                                                let type=src.type.split('/')[0];
                                                img.title=src.name || '';
                                                if(type==='image'){
                                                    src=URL.createObjectURL(src);
                                                    blobs.push(src);
                                                }
                                                else{
                                                    if(type!=='video' && type!=='audio'){
                                                        type='document';
                                                    }
                                                    src=Themify.includesURL+'images/media/'+type+'.png';
                                                }
                                            }
                                            else{
                                                hasDownLoad=true;
                                            }
                                            img.className='tf_box';
                                            img.decoding='async';
                                            img.src=src;
                                            try{
                                                await img.decode();
                                            }
                                            catch(e){
                                            }
                                            img_wrap.appendChild(img); 
                                        }
                                        if(summ>len){
                                            summ=len;
                                        }
                                        msg.innerHTML=hasDownLoad?downLoadMsg.replaceAll('%from%',summ):upLoadMsg.replaceAll('%from%',summ);
                                        f.append(msg,img_wrap);
                                        await TF_Notification.show('info',f);
                                    }
                                    if(images[i+1]!==und){
                                        try{
                                            downLoadNext(images[i+1]);
                                        }
                                        catch(e){

                                        }
                                    }
                                    try{
                                        res=await this._batchDownload(imgArr,timer);
                                    }
                                    catch(e){

                                    }
                                }
                                
                                let uploadData= _prepareUpload(images[i],res),
                                    formData=uploadData.formData;
                                if(isVisible===true && uploadData.skip.size>0){
                                    let msg=[...uploadData.skip].join(', ');                            
                                    await TF_Notification.showHide('error',download_fail.replaceAll('%post%',msg),2000);
                                }
                                if(found===true && res.length>0){
                                    delete uploadData.skip;
                                    if(isVisible===true && upLoadMsg!==''){
                                        let msg=TF_Notification.el.shadowRoot.querySelector('.msg_text');
                                        if(msg){
                                            if(summ>len){
                                                summ=len;
                                            }
                                            msg.innerHTML=upLoadMsg.replaceAll('%from%',summ);
                                        }
                                    }
                                    try{
                                        isSend1=true;
                                        await (new Promise(resolve=>{
                                            setTimeout(async()=>{
                                                let r;
                                                try{
                                                    r=await this._batchUpload(formData,timer);
                                                }
                                                catch(e){
                                                    
                                                }
                                                if(r!==null){
                                                    errors=new Map([...errors,..._uploadFinall(r,imgArr)]);
                                                }
                                                resolve();
                                            },(isSend2===true?1000:10));
                                        }));
                                    }
                                    catch(e){

                                    }
                                }
                            }
                            catch(e){
                                
                            }
                        }
                        for(let j=blobs.length-1;j>-1;--j){
                            URL.revokeObjectURL(blobs[j]);
                        }
                        blobs=[];
                        resolve();
                    });
                    if(images.length>1){
                        prms[1]=new Promise(async resolve =>{
                            for(let i=images.length-1;i>-1;--i){
                                try{
                                let res=[],
                                    found=false,
                                    imgArr=_getImagesUrls(images[i]);
                                    for(let j=imgArr.length-1;j>-1;--j){
                                        if(!resp.has(imgArr[j])){
                                            found=true;
                                            break;
                                        }
                                    }
                                    if(found===true){
                                        isSend2=false;
                                        if(images[i-1]!==und){
                                            try{
                                                downLoadNext(images[i-1]);
                                            }
                                            catch(e){

                                            }
                                        }
                                        try{
                                            res=await this._batchDownload(imgArr,timer);
                                        }
                                        catch(e){

                                        }
                                        isSend2=true;
                                        await (new Promise(resolve=>{
                                            setTimeout(async()=>{
                                                let uploadData=_prepareUpload(images[i],res),
                                                    r;

                                                if(res.length>0){
                                                    try{
                                                        r=await this._batchUpload(uploadData.formData,timer);
                                                    }
                                                    catch(e){

                                                    }
                                                    if(r!==null){
                                                        errors=new Map([...errors,..._uploadFinall(r,imgArr)]);
                                                    }
                                                }
                                                summ+=imgArr.length;
                                                
                                                resolve();
                                            },(isSend1===true?1300:10));
                                        }));
                                    }
                                }
                                catch(e){

                                }
                            }
                            resolve();
                        });
                    }
                    this.downloaded.clear();
                    this.uploading.clear();
                    win.tfOff('beforeunload',preventReload)
                    .document.tfOff('visibilitychange', visibilitychange, {passive:true});
                    await Promise.all(prms);
                    this.uploading.clear();
                    if(isVisible===true && errors.size>0){
                        const size=errors.size,
                            time=size>15?300:(size>10?500:size>3?1000:2000);      
                        for(let [url,msg] of errors){  
                            await TF_Notification.showHide('error',upload_fail.replaceAll('%post%',url).replaceAll('%msg%',msg),time);
                        }
                    }
                    errors.clear();
                    resolve(resp);
            });
        }
    };
})( Themify,document,window.top,Promise,undefined);