((api, doc) => {
    'use strict';
    api.ModuleStar = class extends api.Module {
        constructor(fields) {
            super(fields);
        }
        static getOptions() {
            return [
                {
                    id: 'm_t',
                    type: 'title'
                },
                {
                    id: 'rates',
                    type: 'builder',
                    options: [
                        {
                            type: 'text',
                            label: 'tbefore',
                            id: 'text_b',
                            control: {
                                selector: '.tb_star_text_b'
                            }
                        },
                        {
                            type: 'text',
                            label: 'tafter',
                            id: 'text_a',
                            control: {
                                selector: '.tb_star_text_a'
                            }
                        },
                        {
                            type: 'icon',
                            label: 'icon',
                            default:'fas fullstar',
                            id: 'ic'
                        },
                        {
                            id: 'count',
                            type: 'range',
                            label: 'strc',
                            default:5,
                            min: 1,
                            max: 20
                        },
                        {
                            id: 'rating',
                            type: 'slider_range',
                            label: 'rating',
                            default:5,
                            options: {
                                max: 20,
                                step: .1,
                                unit: '',
                                inputRange: true,
                                range: false
                            }
                        }
                    ]
                },
                {
                    type: 'custom_css_id',
                    custom_css: 'css'
                }
            ];
        }
        static default() {
            return {
                    rates: [{}]
            };
        }
        static builderSave(settings) {
            const rates=settings.rates;
                
            if(rates){
                for(let i=rates.length-1;i>-1;--i){
                    let rate=rates[i];
                    if(rate.count!==undefined && parseInt(rate.count)===5){
                        delete rate.count;
                    }
                    if(rate.rating!==undefined && parseInt(rate.rating)===5){
                        delete rate.rating;
                    }
                    if(rate.ic==='fas fullstar'){
                        delete rate.ic;
                    }
                }
            }
            super.builderSave(settings);
        }
        preview(data) {
            const module = doc.createElement('div'),
                starWrap = doc.createElement('div'),
                    rates=data.rates || [],
                    classes = ['module', 'module-star'],
                    ns = 'http://www.w3.org/2000/svg';

            if (data.css) {
                classes.push(data.css);
            }

            module.className = classes.join(' ');
            starWrap.className='tb_star_wrap';
            for(let i=0;i<rates.length;++i){
                let item=rates[i],
                    count =parseInt(item.count) || 5,
                    rating = parseFloat(parseFloat(item.rating || count).toFixed(2)),
                    starItem=doc.createElement('div'),
                    starContainer=doc.createElement('div');
                    
                    starItem.className='tb_star_item tb_star_animate';
                    starContainer.className='tb_star_container';
                    
                    if(item.text_b){
                        let textBefore=doc.createElement('span');
                        textBefore.className='tb_star_text_b';
                        this.setEditableContent(textBefore,'text_b',item.text_b,'','rates',i);
                        starItem.appendChild(textBefore);
                    }
                    for(let j=0;j<count;++j){
                        let icon=api.Helper.getIcon((item.ic || 'fas fullstar')),
                            cl=icon.classList;
                        if((rating-j)>=1){
                            cl.add('tb_star_fill');
                        }
                        else if(rating>j){
                            let gid='tb_'+data.cid+i,
                                decimal =(rating-parseInt(rating)).toFixed(2),
                                svg = doc.createElementNS(ns, 'svg'),
                                defs= doc.createElementNS(ns,'defs'),
                                linearGradient= doc.createElementNS(ns,'linearGradient'),
                                fill= doc.createElementNS(ns,'stop'),
                                currentColor= doc.createElementNS(ns,'stop');
                                
                            cl.add('tb_star_half');
                            cl.remove('tb_star_fill');
                            icon.style.setProperty('--tb_star_half','url(#'+gid+')');
                            
                            svg.setAttribute('width', 0);
                            svg.setAttribute('height',0);
                            svg.setAttribute('aria-hidden', 'true');
                            svg.style.visibility='hidden';
                            svg.style.position='absolute';
                            
                            linearGradient.id=gid;
                            
                            fill.setAttribute('offset',(decimal*100)+'%');
                            fill.setAttribute('class', 'tb_star_fill');
                            currentColor.setAttribute('offset',(decimal*100)+'%');
                            currentColor.setAttribute('stop-color','currentColor');
                            
                            linearGradient.append(fill,currentColor);
                            defs.appendChild(linearGradient);
                            svg.appendChild(defs);
                            
                            starContainer.appendChild(svg);
                        }
                        starContainer.appendChild(icon);
                    }
                    
                    starItem.appendChild(starContainer);
                    
                    if(item.text_a){
                        let textAfter=doc.createElement('span');
                        textAfter.className='tb_star_text_a';
                        this.setEditableContent(textAfter,'text_a',item.text_a,'','rates',i);
                        starItem.appendChild(textAfter);
                    }
                    starWrap.appendChild(starItem);
                    
            }
            if (data.m_t) {
                module.appendChild(this.getModuleTitle(data.m_t,'m_t'));
            }
            module.appendChild(starWrap);
            return module;
        }
    };
})(tb_app, document);