((api, doc,und) => {
    'use strict';
    api.ModuleTestimonialSlider = class extends api.Module {
        constructor(fields) {
            super(fields);
        }
        static getOptions() {
            return [
                {
                    id: 'mod_title_testimonial',
                    type: 'title'
                },
                {
                    id: 'tab_content_testimonial',
                    type: 'builder',
                    options: [
                        {
                            id: 'title_testimonial',
                            type: 'text',
                            label: 'testimtit',
                            control: {
                                selector: '.testimonial-title'
                            }
                        },
                        {
                            id: 'content_testimonial',
                            type: 'wp_editor',
                            control: {
                                selector: '.testimonial-entry-content'
                            }
                        },
                        {
                            id: 'person_picture_testimonial',
                            type: 'image',
                            label: 'perspic'
                        },
                        {
                            id: 'person_name_testimonial',
                            type: 'text',
                            label: 'persname',
                            control: {
                                selector: '.person-name'
                            }
                        },
                        {
                            id: 'person_position_testimonial',
                            type: 'text',
                            label: 'perspos',
                            control: {
                                selector: '.person-position'
                            }
                        },
                        {
                            id: 'company_testimonial',
                            type: 'text',
                            label: 'company',
                            control: {
                                selector: '.person-company'
                            }
                        },
                        {
                            id: 'company_website_testimonial',
                            type: 'text',
                            label: 'compweb'
                        },
                        {
                            type: 'multi',
                            label: 'rstar',
                            options: [
                                {
                                    type: 'icon',
                                    label: 'icon',
                                    id: 'ic',
                                    binding: {
                                        empty: {
                                            hide: 'tb_rate_wrap'
                                        },
                                        not_empty: {
                                            show: 'tb_rate_wrap'
                                        }
                                    }
                                },
                                {
                                    id: 'count',
                                    type: 'range',
                                    wrap_class: 'tb_rate_wrap',
                                    label: 'strc',
                                    default:5,
                                    min: 1,
                                    max: 20
                                },
                                {
                                    id: 'rating',
                                    type: 'slider_range',
                                    wrap_class: 'tb_rate_wrap',
                                    label: 'rating',
                                    default:5,
                                    options: {
                                        max: 20,
                                        step: 0.1,
                                        unit: '',
                                        inputRange: true,
                                        range: false
                                    }
                                }
                            ]
                        }
                    ]
                },
                {
                    id: 'type_testimonial',
                    type: 'radio',
                    label: 'type',
                    options: [
                        {
                            value: 'slider',
                            name: 'slider'
                        },
                        {
                            value: 'grid',
                            name: 'grid'
                        }
                    ],
                    option_js: true,
                    binding : {
                        slider : { hide : 'items_per_slide' }
                    }
                },
                {
                    id: 'grid_layout_testimonial',
                    type: 'layout',
                    label: 'gridl',
                    mode: 'sprite',
                    post_grid: true,
                    default:'list-post',
                    binding: {
                        grid4: {
                            show: 'masonry'
                        },
                        grid3: {
                            show: 'masonry'
                        },
                        grid2: {
                            show: 'masonry'
                        },
                        'list-post': {
                            hide: 'masonry'
                        }
                    },
                    control: {
                        classSelector: '.themify_builder_testimonial'
                    },
                    wrap_class: 'tb_group_element_grid'
                },
                {
                    id: 'masonry',
                    type: 'toggle_switch',
                    label: 'masnry',
                    options: {
                        on: {
                            name: 'enable',
                            value: 'en'
                        },
                        off: {
                            name: 'disable',
                            value: 'dis'
                        }
                    },
                    wrap_class: 'tb_group_element_grid'
                },
                {
                    id: 'layout_testimonial',
                    type: 'layout',
                    label: 'lay',
                    mode: 'sprite',
                    default:'image-top',
                    options: [
                        {
                            img: 'testimonials_image_top',
                            value: 'image-top',
                            label: 'imgtop'
                        },
                        {
                            img: 'testimonials_image_bottom',
                            value: 'image-bottom',
                            label: 'imgbtm'
                        },
                        {
                            img: 'testimonials_image_bubble',
                            value: 'image-bubble',
                            label: 'imgbubl'
                        }
                    ],
                    control: {
                        classSelector: '.themify_builder_slider_wrap'
                    }
                },
                {
                    id: 'img_w_slider',
                    type: 'number',
                    label: 'imgw',
                    after: 'px'
                },
                {
                    id: 'img_h_slider',
                    type: 'number',
                    label: 'imgh',
                    after: 'px'
                },
                {
                    id: 'slider_option_testimonial',
                    type: 'slider',
                    wrap_class: 'tb_group_element_slider'
                },
                {
                    type: 'custom_css_id',
                    custom_css: 'css_testimonial'
                }
            ];
        }
        static default() {
            return {
                img_h_slider: 100,
                img_w_slider: 100,
                tab_content_testimonial: [
                    {
                        title_testimonial: 'Optional Title',
                        content_testimonial: themifyBuilder.i18n.label.testimcont,
                        person_name_testimonial: 'John Smith',
                        person_position_testimonial: 'CEO',
                        company_testimonial: 'X-corporation',
                        ic: 'fas fullstar'
                    }
                ]
            };
        }
        static builderSave(settings) {
            const def = {
                type_testimonial:'slider',
                layout_testimonial: 'image-top',
                grid_layout_testimonial:'list-post'
            },
            testominals=settings.tab_content_testimonial;
            for (let key in def) {
                if (settings[key] === def[key]) {
                    delete settings[key];
                }
            }
            delete settings.items_per_slide;
            if(!settings.grid_layout_testimonial){
                delete settings.masonry;
            }
            this.clearSliderOptions(settings,!!settings.type_testimonial);
            if(testominals){
                for(let i=testominals.length-1;i>-1;--i){
                    let testominal=testominals[i];
                    if(!testominal.ic || (testominal.count!==und && parseInt(testominal.count)===5)){
                        delete testominal.count;
                    }
                    if(!testominal.ic ||testominal.rating!==und && parseInt(testominal.rating)===5){
                        delete testominal.rating;
                    }
                    if(!testominal.person_name_testimonial){
                        delete testominal.person_position_testimonial;
                        delete testominal.company_testimonial;
                    }
                    if(!testominal.company_testimonial){
                        delete testominal.company_website_testimonial;
                    }
                }
            }
            super.builderSave(settings);
        }
        getItem(item, data,index,isSlider) {
            const post = doc.createElement('div'),
                testimonial = doc.createElement('div'),
                content = doc.createElement('div'),
                deleteBtn=doc.createElement('div'),
                repeatId='tab_content_testimonial';
                
            post.className='post';
            testimonial.className='testimonial-item tf_rel';
            content.className='testimonial-content';
            if (item.title_testimonial){
                const title=doc.createElement('h3');
                title.className='testimonial-title';
                this.setEditableContent(title,'title_testimonial',item.title_testimonial,'',repeatId,index);
                content.appendChild(title);
            }
            if(item.ic){
                const ratingWrap=doc.createElement('div'),
                    count =parseInt(item.count) || 5,
                    rating = parseFloat(parseFloat(item.rating || count).toFixed(2)),
                    ns = 'http://www.w3.org/2000/svg';
                ratingWrap.className='tb_rating_wrap';
                for(let j=0;j<count;++j){
                    let icon=api.Helper.getIcon(item.ic),
                            cl=icon.classList;
                        if((rating-j)>=1){
                            cl.add('tb_rating_fill');
                        }
                        else if(rating>j){
                            let gid='tb_'+data.cid+index,
                                decimal =(rating-parseInt(rating)).toFixed(2),
                                svg = doc.createElementNS(ns, 'svg'),
                                defs= doc.createElementNS(ns,'defs'),
                                linearGradient= doc.createElementNS(ns,'linearGradient'),
                                fill= doc.createElementNS(ns,'stop'),
                                currentColor= doc.createElementNS(ns,'stop');
                                
                            cl.add('tb_rating_half');
                            cl.remove('tb_rating_fill');
                            icon.style.setProperty('--tb_rating_half','url(#'+gid+')');
                            
                            svg.setAttribute('width', 0);
                            svg.setAttribute('height',0);
                            svg.setAttribute('aria-hidden', 'true');
                            svg.style.visibility='hidden';
                            svg.style.position='absolute';
                            
                            linearGradient.id=gid;
                            
                            fill.setAttribute('offset',(decimal*100)+'%');
                            fill.setAttribute('class', 'tb_rating_fill');
                            currentColor.setAttribute('offset',(decimal*100)+'%');
                            currentColor.setAttribute('stop-color','currentColor');
                            
                            linearGradient.append(fill,currentColor);
                            defs.appendChild(linearGradient);
                            svg.appendChild(defs);
                            
                            ratingWrap.appendChild(svg);
                        }
                        ratingWrap.appendChild(icon);
                }
                content.appendChild(ratingWrap);
            }
            if (item.content_testimonial){
                const contentTestimonial=doc.createElement('div');
                contentTestimonial.className='testimonial-entry-content';
                this.setEditableContent(contentTestimonial,'content_testimonial','',1,repeatId,index);
                if(item.content_testimonial){
                    contentTestimonial.innerHTML=this.shortcodeToHTML(item.content_testimonial).content;
                }
                content.appendChild(contentTestimonial);
            }
            
            if (item.person_picture_testimonial) {
                const img=new Image(),
                    figure=doc.createElement('figure');
                this.setEditableImage(img,'person_picture_testimonial','img_w_slider','img_h_slider',data,repeatId,index);
                img.src=item.person_picture_testimonial;
                img.title=item.title_testimonial || '';
                figure.className='testimonial-image tf_rel';
                figure.appendChild(img);
                content.appendChild(figure);
            }
            
            if (item.person_name_testimonial){
                const author=doc.createElement('div'),
                    name=doc.createElement('div');
                author.className='testimonial-author';
                name.className='person-name';
                this.setEditableContent(name,'person_name_testimonial',item.person_name_testimonial,'',repeatId,index);
                author.appendChild(name);
                
                if (item.person_position_testimonial){
                    const pos=doc.createElement('div');
                    pos.className='person-position';
                    this.setEditableContent(pos,'person_position_testimonial',item.person_position_testimonial,'',repeatId,index);
                    author.appendChild(pos);
                }
                if (item.company_testimonial){
                    let company=doc.createElement('div'),
                        editItem=company;
                    company.className='person-company';
                    
                    if(item.company_website_testimonial){
                        editItem=doc.createElement('a');
                        editItem.href=item.company_website_testimonial;
                        company.appendChild(editItem);
                    }
                    this.setEditableContent(editItem,'company_testimonial',item.company_testimonial,'',repeatId,index);
                    
                    author.appendChild(company);
                }
                content.appendChild(author);
            }
            
            if(isSlider){
                post.className+=' tf_swiper-slide';
                if (data.left_margin_slider>0) {
                    testimonial.style.marginLeft=data.left_margin_slider+'px';
                }
                if (data.right_margin_slider>0) {
                    testimonial.style.marginRight=data.right_margin_slider+'px';
                }
            }
            deleteBtn.title = 'Delete Testimonial';
            deleteBtn.role='button';
            deleteBtn.tfOn(Themify.click, e => {
                e.stopPropagation();
                e.preventDefault();
                let item = e.currentTarget.closest('.post'),
                        index = item.dataset.swiperSlideIndex??Themify.convert(item.parentNode.tfClass('post')).indexOf(item);
                if (index !== -1) {
                    index=parseInt(index);
                    if (api.activeModel?.id === this.id) {
                        Themify.triggerEvent(api.LightBox.el.tfClass('tb_delete_row')[index], e.type);
                    } 
                    else {
                        api.undoManager.start('inlineDelete');
                        const settings = this.get('mod_settings');
                        settings.tab_content_testimonial.splice(index, 1);
                        this.set('mod_settings', settings);
                        if(!settings.type_testimonial || settings.type_testimonial==='slider'){
                            this.previewLive(settings);
                        }else{
                            item.remove();
                        }
                        api.undoManager.end('inlineDelete');
                    }
                }
            })
            .className = 'tb_del_btn tb_del_tstimbtn tf_close tb_disable_sorting';

            testimonial.append(content,deleteBtn);
            post.appendChild(testimonial);

            return post;
        }
        preview(data) {
            let arr=data.tab_content_testimonial || [],
                isSlider=!data.type_testimonial || data.type_testimonial==='slider',
                module = doc.createElement('div'),
                wrap= doc.createElement('div'),
                addBtn = doc.createElement('div'),
                classes = ['module', 'module-testimonial-slider','tf_clearfix'],
                itemsWrap=wrap,
                dataset=wrap.dataset;
            if(isSlider){
                classes.push('themify_builder_slider_wrap');
                if(data.img_fullwidth_slider==='fullwidth'){
                    classes.push('slide-image-fullwidth');
                }
                if(data.show_arrow_buttons_vertical==='vertical' && (!data.show_arrow_slider || data.show_arrow_slider==='yes')){
                    classes.push('slide-image-themify_builder_slider_vertical');
                    dataset.nav_out=1;
                }
            }
            if (data.layout_slider) {
                classes.push(data.layout_slider);
            }
            if (data.css_slider) {
                classes.push(data.css_slider);
            }
            
            module.className = classes.join(' ');
            if(!isSlider){
                wrap.className='themify_builder_testimonial loops-wrapper builder-posts-wrap tf_rel tf_clear';
                if(data.grid_layout_testimonial){
                    wrap.className+=' '+data.grid_layout_testimonial;
                }
                if(data.masonry==='enable'){
                    wrap.className+=' masonry';
                }
            }
            else{
                const opt={...{
					open_link_new_tab_slider:'no',
					pause_on_hover_slider:'resume',
					unlink_feat_img_slider:'no',
					unlink_post_title_slider:'no',
                    auto_scroll_opt_slider:'off',
                    scroll_opt_slider:1,
                    speed_opt_slider:1
				},...data},
                cssUrl=ThemifyBuilderModuleJs.cssUrl+'sliders/',
                isHorizontal=opt.horizontal=== 'yes';
                
                wrap.className='themify_builder_slider tf_carousel tf_swiper-container tf_rel tf_overflow';
                dataset.tbreakpoints=api.Utils.getBPWidth('tablet_landscape');
                dataset.mbreakpoints=api.Utils.getBPWidth('mobile');
                dataset.visible=isHorizontal ? 4 : (opt.visible_opt_slider || '');
                dataset.scroll=opt.scroll_opt_slider;
                if(opt.tab_visible_opt_slider){
                    dataset.tabVisible=opt.tab_visible_opt_slider;
                }
                if(opt.mob_visible_opt_slider){
                    dataset.mobVisible=opt.mob_visible_opt_slider;
                }
                if(opt.auto_scroll_opt_slider!=='off'){
                    dataset.auto=opt.auto_scroll_opt_slider*1000;
                    dataset.controller=opt.play_pause_control=== 'yes'?1:0;
                    dataset.pause_hover=opt.pause_on_hover_slider=== 'resume'?1:0;
                }
                dataset.speed=opt.speed_opt_slider === 'slow' ? 4 : (opt.speed_opt_slider === 'fast' ? .5 : opt.speed_opt_slider);
                dataset.wrapvar=opt.wrap_slider!== 'no'?1:0;
                dataset.slider_nav=opt.show_arrow_slider!== 'no'?1:0;
                dataset.pager=opt.show_nav_slider!== 'no'?1:0;
                dataset.effect=opt.effect_slider || 'scroll';
                dataset.height=isHorizontal || !opt.height_slider? 'variable' : opt.height_slider;
                dataset.horizontal=isHorizontal?1:0;
                dataset.css_url=cssUrl+'carousel,'+cssUrl+'slider';
                if(opt.touch_swipe){
                    dataset.touch_swipe=opt.touch_swipe;
                }
                
                itemsWrap=doc.createElement('div');
                itemsWrap.className='tf_swiper-wrapper tf_lazy tf_rel tf_w tf_h tf_textc';
                wrap.appendChild(itemsWrap);
            }
            for (let i = 0; i < arr.length; ++i) {
                itemsWrap.appendChild(this.getItem(arr[i], data,i,isSlider));
            }
            addBtn.title = 'Add Testimonial';
            addBtn.role='button';
            addBtn.tfOn(Themify.click, e => {
                e.stopPropagation();
                if (api.activeModel?.id === this.id) {
                    Themify.triggerEvent(api.LightBox.el.tfClass('add_new')[0], e.type);
                } else {
                    api.undoManager.start('inlineAdd');
                    const settings = this.get('mod_settings'),
                        def = this.constructor.default().tab_content_testimonial?.[0] || {};
                    if (!settings.tab_content_testimonial) {
                        settings.tab_content_testimonial = [];
                    }
                    settings.tab_content_testimonial.push(def);
                    this.set('mod_settings', settings);
                    if(!settings.type_testimonial || settings.type_testimonial==='slider'){
                        this.previewLive(settings);
                    }
                    else{
                        e.currentTarget.parentNode.tfClass('builder-posts-wrap')[0].appendChild(this.getItem(def, settings));
                    }
                    api.undoManager.end('inlineAdd');
                }

            }, {passive: true})
            .className = 'tb_add_btn tb_add_tstimbtn tf_plus_icon tb_disable_sorting';
            
            if (data.mod_title_slider) {
                module.appendChild(this.getModuleTitle(data.mod_title_slider,'mod_title_slider'));
            }
            module.append(wrap,addBtn);
            return module;
        }
    };
})(tb_app, document,undefined);