/**
 * signup module
 */

(( Themify, window ) => {
    'use strict';

	function init( form ) {
		form.addEventListener( 'submit', async ( e ) => {
			e.preventDefault();

			const submitBtn = form.querySelector( 'button[name="tb_submit"]' ),
				errorEl = form.tfClass( 'tb_signup_errors' )[0],
				successEl = form.tfClass( 'tb_signup_success' )[0];
			submitBtn.disabled = true;
			errorEl.replaceChildren();
			if ( successEl ) {
				successEl.style.display = 'none';
			}

			try {
				const data = new FormData(form);
				data.append( 'action', 'tb_signup_process' );
				const result = await Themify.fetch( data );
				if ( result.success ) {
					if ( successEl ) {
						successEl.style.display = 'block';
					}
					const redirect = form.dataset.redirect;
					if ( redirect === '' ) {
						window.location.reload( true );
					} else if ( redirect ) {
						window.location.href = redirect;
					}
				} else {
					throw new Error( result.data );
				}
			} catch( e ) {
				errorEl.innerHTML = e.message ? e.message : form.dataset.genericError;
				submitBtn.disabled = false;
			}
		} );

		const cpg = form.tfClass('themify_captcha_field g-recaptcha')[0],
            cph = form.tfClass('themify_captcha_field h-captcha')[0];
        if( cpg || cph ) {
            if (cpg && typeof grecaptcha === 'undefined') {
                const key=cpg.dataset.sitekey;
                if(key){
                    let url = 'https://www.google.com/recaptcha/api.js';
                    if( 'v3' === cpg.dataset.ver){
                        url+='?render='+key;
                    }
                    Themify.loadJs(url,typeof grecaptcha !== 'undefined',false);
                }
            }
            if ( cph && typeof window.hcaptcha === 'undefined' ) {
                const key = cph.dataset.sitekey;
                if ( key ) {
                    let url = 'https://js.hcaptcha.com/1/api.js';
                    Themify.loadJs(url,typeof window.hcaptcha !== 'undefined',false);
                }
            }
        }
	}

    Themify.on('builder_load_module_partial', ( el, isLazy ) => {
        if ( isLazy === true && ! el.classList.contains( 'module-signup-form' ) ) {
            return;
        }
        const forms = Themify.selectWithParent( 'tb_signup_form', el );
		for( let i = forms.length - 1; i > -1; --i ) {
			init( forms[ i ] );
		}
    });

})( Themify, window );